/**
 * JASMINe
 * Copyright (C) 2006-2007 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.configurator.statik.velocity;

import java.util.Properties;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.objectweb.jasmine.configurator.AbstractConfiguratorComposite;
import org.objectweb.jasmine.util.JasmineWrapperUtil;

/**
 * @author Julien Legrand
 *
 */
public abstract class AbstractVelocityCompositeConfigurator extends
        AbstractConfiguratorComposite implements VelocityConfigurator {

    /**
     *
     */
    protected VelocityEngine ve = null;

    /**
     *
     */
    protected VelocityContext context = null;

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    public AbstractVelocityCompositeConfigurator() {

        ve = new VelocityEngine();
        context = new VelocityContext();
        context.put("this", this);

    }

    // ------------------------------------------------------------------------
    // Implementation of the VelocityConfigurator interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.VelocityConfigurator#getContext()
     */
    public VelocityContext getContext() {
        return this.context;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.statik.velocity.VelocityConfigurator#getGeneratedFileName()
     */
    public String getGeneratedFileName() {
        return null;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.statik.velocity.VelocityConfigurator#getTemplateFile()
     */
    public String getTemplateFile() {
        return null;
    }

    // ------------------------------------------------------------------------
    // Implementation of the Configurator interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.Configurator#getProperties()
     */
    public Properties getProperties() {

        Properties props = super.getProperties();

        Properties myprops = new Properties();
        for (Object key : context.getKeys()) {
            myprops.put(key, context.get(key.toString()));
        }

        JasmineWrapperUtil.concatProperties(props, myprops);

        return props;

    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.Configurator#getProperty(java.lang.String)
     */
    public Object getProperty(String key) {

        Object o = super.getProperty(key);

        if (o == null) {
            return context.get(key);
        } else {

            return o;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.Configurator#setProperty(java.lang.String,
     *      java.lang.Object)
     */
    public boolean setProperty(String key, Object value) {

        if (!super.setProperty(key, value)) {
            context.put(key, value);
            return true;
        }
        return true;
    }

}
