/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.configurator.statik.velocity;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.StringWriter;
import java.util.Properties;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.objectweb.jasmine.configurator.ConfigurationException;

/**
 * @author Julien Legrand
 *
 */
public class VelocityConfiguratorImpl implements VelocityConfigurator {

    /**
     *
     */
    private VelocityEngine ve = null;

    /**
     *
     */
    private VelocityContext context = null;

    /**
     *
     */
    private Template template = null;

    /**
     *
     */
    private String templateName = null;

    /**
     *
     */
    private String fileName = null;

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    public VelocityConfiguratorImpl() {

        ve = new VelocityEngine();
        context = new VelocityContext();
        context.put("this", this);
    }

    public VelocityConfiguratorImpl(ClassLoader cl) {

        try {

            ve = new VelocityEngine();
            context = new VelocityContext();
            context.put("this", this);

            ve.setProperty("resource.loader", "class");
            ve
                    .setProperty("class.resource.loader.class",
                            "org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader");
            if (cl != null)
                ve.setProperty("class.resource.loader.classloader", cl);

            ve.init();

        } catch (ResourceNotFoundException rnfe) {
            System.out.println("cannot find template " + getTemplateFile());
        } catch (ParseErrorException pee) {
            System.out.println("Syntax error in template " + getTemplateFile()
                    + ":" + pee);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public VelocityConfiguratorImpl(VelocityContext vcontext, ClassLoader cl) {

        try {

            ve = new VelocityEngine();
            context = new VelocityContext(vcontext);
            context.put("this", this);

            ve.setProperty("resource.loader", "class");
            ve
                    .setProperty("class.resource.loader.class",
                            "org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader");
            if (cl != null)
                ve.setProperty("class.resource.loader.classloader", cl);

            ve.init();

        } catch (ResourceNotFoundException rnfe) {
            System.out.println("cannot find template " + getTemplateFile());
        } catch (ParseErrorException pee) {
            System.out.println("Syntax error in template " + getTemplateFile()
                    + ":" + pee);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public VelocityConfiguratorImpl(VelocityContext vcontext,
            ClassLoader cl, String templateName, String fileName) {

        try {

            ve = new VelocityEngine();
            context = new VelocityContext(vcontext);
            context.put("this", this);

            ve.setProperty("resource.loader", "class");
            ve
                    .setProperty("class.resource.loader.class",
                            "org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader");
            if (cl != null)
                ve.setProperty("class.resource.loader.classloader", cl);

            ve.init();

            this.templateName = templateName;
            this.fileName = fileName;

        } catch (ResourceNotFoundException rnfe) {
            System.out.println("cannot find template " + getTemplateFile());
        } catch (ParseErrorException pee) {
            System.out.println("Syntax error in template " + getTemplateFile()
                    + ":" + pee);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // ------------------------------------------------------------------------
    // Implementation of the Configurator interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.Configurator#configure()
     */
    public void configure() throws ConfigurationException {

        try {

            write(getGeneratedFileName());

        } catch (ResourceNotFoundException e) {
            throw new ConfigurationException(e);
        } catch (ParseErrorException e) {
            throw new ConfigurationException(e);
        } catch (Exception e) {
            throw new ConfigurationException(e);
        }

    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.Configurator#put(java.lang.String,
     *      java.lang.Object)
     */
    public boolean setProperty(String key, Object value) {
        context.put(key, value);
        return true;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.Configurator#get(java.lang.String)
     */
    public Object getProperty(String key) {
        return context.get(key);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.Configurator#getProperties()
     */
    public Properties getProperties() {

        Properties properties = new Properties();

        for (Object key : context.getKeys()) {

            properties.put(key, context.get(key.toString()));

        }

        return properties;
    }

    // ------------------------------------------------------------------------
    // Implementation of the VelocityConfigurator interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.VelocityConfigurator#getContext()
     */
    public VelocityContext getContext() {
        return this.context;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.statik.velocity.VelocityConfigurator#getGeneratedFileName()
     */
    public String getGeneratedFileName() {

        String installDir = context.get("install-dir").toString();
        if(installDir != null){
            File file = new File(installDir);
            if(file.exists() && file.isDirectory()){
                return installDir + File.separator + fileName;
            }
        }

        return fileName;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.statik.velocity.VelocityConfigurator#getTemplateFile()
     */
    public String getTemplateFile() {
        return templateName;
    }

    // ------------------------------------------------------------------------
    // Private methods
    // ------------------------------------------------------------------------

    /**
     * Write generated configuration in the file "filename"
     *
     * @param filename
     * @throws Exception
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     */
    private void write(String filename) throws ResourceNotFoundException,
            ParseErrorException, Exception {

        File file = new File(filename);
        if (!file.exists())
            file.createNewFile();

        BufferedWriter out = new BufferedWriter(new FileWriter(filename));

        StringWriter writer = new StringWriter();

        template = ve.getTemplate(getTemplateFile());
        if (template != null)
            template.merge(context, writer);

        /**
         * TODO : understand why there's spaces at the begin of the buffer !
         */
        String buffer = writer.getBuffer().toString();
        while(buffer.startsWith(" "))
            buffer = buffer.replaceFirst(" ", "");

        out.write(buffer);
        out.close();
    }
}
