/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.configurator.statik.velocity.templategenerator;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;

/**
 * @author Julien Legrand
 *
 */
public abstract class AbstractTemplateGenerator implements TemplateGenerator {

    /**
     *
     */
    protected String prefix;

    /**
     *
     */
    protected Set<String> templateVars = new HashSet<String>();

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    public AbstractTemplateGenerator(String prefix) {
        this.prefix = prefix;
    }

    // ------------------------------------------------------------------------
    // Implementation of the TemplateGenerator interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.statik.velocity.templategenerator.TemplateGenerator#getProperties()
     */
    public Set<String> getProperties() {
        return templateVars;
    }

    // ------------------------------------------------------------------------
    // Private methods
    // ------------------------------------------------------------------------

    /**
     * @param key
     * @param value
     * @return
     */
    protected List<String[]> proceedProperty(String key, String value) {

        List<String[]> res = new ArrayList<String[]>();

        String prfx = format(key);

        /*
         * proceed URL
         */
        if (value.contains("://")) {

            StringTokenizer tokenizer = new StringTokenizer(value, "://");

            String protocol = tokenizer.nextToken();
            res.add(new String[] { prfx + "-protocol", protocol });

            if (tokenizer.hasMoreTokens()) {

                String address = tokenizer.nextToken();

                res.add(new String[] { prfx + "-host", address });

                if (tokenizer.hasMoreTokens()) {

                    String port = tokenizer.nextToken();

                    res.add(new String[] { prfx + "-port", port });
                }
            }

        } else {

            res.add(new String[] { prfx, value });

        }

        return res;
    }

    /**
     * @param key
     * @return
     */
    protected String format(String key) {

        return prefix + "-" + key.replace(".", "-");
    }

    /**
     * @param var
     * @param value
     * @return
     */
    protected String getDefaultPattern(String var, String value) {

        return DEFAULT_INITIALISATION_PATTERN.replace("$", "$" + var).replace(
                "\"\"", "\"" + value + "\"");
    }
}
