/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.configurator.statik.velocity.templategenerator;

import java.io.File;
import java.io.FileFilter;
import java.util.Set;

/**
 *
 * @author Julien Legrand
 *
 */
public class GeneratorLauncher {

    /**
     * @param args
     */
    public static void main(String[] args) {

        if (args.length < 1) {
            System.out
                    .println("Usage : java GeneratorLauncher directory_containing_conf ");
        } else {

            String dirName = args[0];
            File dir = new File(dirName);
            if (!dir.isDirectory()) {
                System.err.println(dirName + " is not a directory.");
                System.out
                        .println("Usage : java GeneratorLauncher directory_containing_conf ");
            } else {

                FileFilter xmlFileFilter = new FileFilter() {
                    public boolean accept(File pathname) {
                        return pathname.getName().endsWith(".xml");
                    }
                };

                FileFilter propertiesFileFilter = new FileFilter() {
                    public boolean accept(File pathname) {
                        return pathname.getName().endsWith(".properties");
                    }
                };

                File xmlFile[] = dir.listFiles(xmlFileFilter);
                File propertiesFile[] = dir.listFiles(propertiesFileFilter);

                StringBuffer buffer = new StringBuffer();

                XMLTemplateGenerator xtg = null;
                File f = null;
                String prefix = null;

                for (int i = 0; i < xmlFile.length; i++) {

                    f = xmlFile[i];
                    prefix = f.getName().replace(".xml", "").replace("-", "_")
                            .toLowerCase();

                    xtg = new XMLTemplateGenerator(prefix);
                    try {
                        System.out.println("Generating " + f.getName() + " ...");

                        xtg.generate(f);

                        buffer.append(setToString(xtg.getProperties()));
                        buffer.append("\n");

                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }

                PropertiesTemplateGenerator ptg = null;

                for (int i = 0; i < propertiesFile.length; i++) {

                    f = propertiesFile[i];
                    prefix = f.getName().replace(".properties", "").replace(
                            "-", "_").toLowerCase();

                    ptg = new PropertiesTemplateGenerator(prefix);
                    try {
                        System.out.println("Generating " + f.getName() + " ...");

                        ptg.generate(f);

                        buffer.append(setToString(ptg.getProperties()));
                        buffer.append("\n");

                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }

                System.out.println("\nGenerated properties : ");
                System.out.println(buffer.toString());

            }
        }
    }

    // ------------------------------------------------------------------------
    // Private methods
    // ------------------------------------------------------------------------

    private static String setToString(Set<String> set){

        StringBuffer buffer = new StringBuffer();
        for(String s : set){
            buffer.append(s);
            buffer.append("\n");
        }

        return buffer.toString();

    }

}
