/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.configurator.statik.velocity.templategenerator;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.List;
import java.util.Properties;

/**
 * This class generates a Velocity template file, used by JASMINe configurator,
 * from a key-value property file.
 *
 * TODO : This class can be optimized.
 *
 * @author Julien Legrand
 *
 */
public class PropertiesTemplateGenerator extends AbstractTemplateGenerator {

    /**
     *
     */
    private final static String PROPERTIES_FILE_EXTENSION = ".properties";

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    public PropertiesTemplateGenerator(String prefix) {
        super(prefix);
    }

    // ------------------------------------------------------------------------
    // Implementation of the TemplateGenerator interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.statik.velocity.templategenerator.TemplateGenerator#generate(java.io.File)
     */
    public void generate(File file) throws Exception {

        /**
         * TODO optimize :-)
         */

        /*
         * create file to generate
         */
        String path = file.getAbsolutePath();
        File templateFile = new File(path.replace(PROPERTIES_FILE_EXTENSION,
                TEMPLATE_EXTENSION));

        /*
         * create BufferedReader
         */
        BufferedReader in = new BufferedReader(new FileReader(file));

        /*
         * create BufferedWriter
         */
        BufferedWriter out = new BufferedWriter(new FileWriter(templateFile));

        /*
         * proceed properties
         */
        Properties properties = new Properties();
        properties.load(new FileInputStream(file));

        StringBuffer buffer = new StringBuffer();

        for (Object key : properties.keySet()) {

            String value = properties.getProperty(key.toString());

            /*
             * define all configurable properties. A property (like URL) can
             * contains several properties.
             */
            List<String[]> list = proceedProperty(key.toString(), value);

            for (String[] s : list) {

                String k = s[0];
                String v = s[1];

                templateVars.add(k);

                buffer.append(getDefaultPattern(k, v));
                buffer.append("\n");
            }

        }

        /*
         * generate file
         */

        out.write(buffer.toString());

        String line;
        String toWrite;
        while ((line = in.readLine()) != null) {

            toWrite = line;

            for (Object key : properties.keySet()) {
                if (line.contains(key.toString())) {

                    String value = properties.get(key.toString()).toString();

                    if (!"".equals(value)) {

                        String prfx = format(key.toString());

                        if (value.contains("://")) {
                            if (value.substring(value.indexOf("://") + 3,
                                    value.length()).contains(":")) {
                                toWrite = replaceLast(line, value, "${" + prfx
                                        + "-protocol}://${" + prfx
                                        + "-host}:${" + prfx + "-port}");
                            } else {
                                toWrite = replaceLast(line, value, "${" + prfx
                                        + "-protocol}://${" + prfx + "-host}");
                            }
                        } else {
                            toWrite = replaceLast(line, value, "${" + prfx
                                    + "}");
                        }
                    }
                    break;
                }
            }

            out.write(toWrite);
            out.write("\n");
        }

        out.close();
        in.close();

    }

    // ------------------------------------------------------------------------
    // Private methods
    // ------------------------------------------------------------------------

    private String replaceLast(String string, String oldChar, String newChar) {

        int idx = string.lastIndexOf(oldChar);

        if (idx == -1)
            return string;

        String firstPart = string.substring(0, idx);
        String secondPart = string.substring(idx, string.length()).replace(
                oldChar, newChar);

        return firstPart + secondPart;

    }

    // ------------------------------------------------------------------------
    // Main method for test
    // ------------------------------------------------------------------------

    public static void main(String[] args) {

        PropertiesTemplateGenerator ptg = new PropertiesTemplateGenerator(
                "carol");

        File jonasProps = new File(
                "/home/legrandj/Desktop/conf/carol.properties");

        try {

            ptg.generate(jonasProps);

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

}
